# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from .mitigations import (
    MitigationsResource,
    AsyncMitigationsResource,
    MitigationsResourceWithRawResponse,
    AsyncMitigationsResourceWithRawResponse,
    MitigationsResourceWithStreamingResponse,
    AsyncMitigationsResourceWithStreamingResponse,
)
from ...pagination import SyncV4PagePagination, AsyncV4PagePagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.abuse_reports import abuse_report_list_params, abuse_report_create_params
from ...types.abuse_reports.abuse_report_get_response import AbuseReportGetResponse
from ...types.abuse_reports.abuse_report_list_response import AbuseReportListResponse
from ...types.abuse_reports.abuse_report_create_response import AbuseReportCreateResponse

__all__ = ["AbuseReportsResource", "AsyncAbuseReportsResource"]


class AbuseReportsResource(SyncAPIResource):
    @cached_property
    def mitigations(self) -> MitigationsResource:
        return MitigationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AbuseReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AbuseReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AbuseReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AbuseReportsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_dmca"],
        address1: str,
        agent_name: str,
        agree: Literal[1],
        city: str,
        country: str,
        email: str,
        email2: str,
        host_notification: Literal["send"],
        name: str,
        original_work: str,
        owner_notification: Literal["send"],
        signature: str,
        state: str,
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          address1: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          agent_name: The name of the copyright holder. Text not exceeding 60 characters. This field
              may be released by Cloudflare to third parties such as the Lumen Database
              (https://lumendatabase.org/).

          agree: Can be `0` for false or `1` for true. Must be value: 1 for DMCA reports

          city: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          country: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          original_work: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          signature: Required for DMCA reports, should be same as Name. An affirmation that all
              information in the report is true and accurate while agreeing to the policies of
              Cloudflare's abuse reports

          state: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_trademark"],
        email: str,
        email2: str,
        host_notification: Literal["send"],
        justification: str,
        name: str,
        owner_notification: Literal["send"],
        trademark_number: str,
        trademark_office: str,
        trademark_symbol: str,
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          justification: A detailed description of the infringement, including any necessary access
              details and the exact steps needed to view the content, not exceeding 5000
              characters.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          trademark_number: Text not exceeding 1000 characters

          trademark_office: Text not exceeding 1000 characters

          trademark_symbol: Text not exceeding 1000 characters

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_general"],
        email: str,
        email2: str,
        host_notification: Literal["send", "send-anon"],
        justification: str,
        name: str,
        owner_notification: Literal["send", "send-anon"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        destination_ips: str | Omit = omit,
        ports_protocols: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        source_ips: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          justification: A detailed description of the infringement, including any necessary access
              details and the exact steps needed to view the content, not exceeding 5000
              characters.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          destination_ips: A list of IP addresses separated by ‘\n’ (new line character). The list of
              destination IPs should not exceed 30 IP addresses. Each one of the IP addresses
              ought to be unique.

          ports_protocols: A comma separated list of ports and protocols e.g. 80/TCP, 22/UDP. The total
              size of the field should not exceed 2000 characters. Each individual
              port/protocol should not exceed 100 characters. The list should not have more
              than 30 unique ports and protocols.

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          source_ips: A list of IP addresses separated by ‘\n’ (new line character). The list of
              source IPs should not exceed 30 IP addresses. Each one of the IP addresses ought
              to be unique.

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_phishing"],
        email: str,
        email2: str,
        host_notification: Literal["send", "send-anon"],
        justification: str,
        name: str,
        owner_notification: Literal["send", "send-anon"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        original_work: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          justification: A detailed description of the infringement, including any necessary access
              details and the exact steps needed to view the content, not exceeding 5000
              characters.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          original_work: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_children"],
        email: str,
        email2: str,
        host_notification: Literal["send", "send-anon"],
        justification: str,
        name: str,
        ncmec_notification: Literal["send", "send-anon"],
        owner_notification: Literal["send", "send-anon", "none"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        country: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          justification: A detailed description of the infringement, including any necessary access
              details and the exact steps needed to view the content, not exceeding 5000
              characters.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          ncmec_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          country: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_threat"],
        email: str,
        email2: str,
        host_notification: Literal["send", "send-anon"],
        justification: str,
        name: str,
        owner_notification: Literal["send", "send-anon"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          justification: A detailed description of the infringement, including any necessary access
              details and the exact steps needed to view the content, not exceeding 5000
              characters.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_registrar_whois"],
        email: str,
        email2: str,
        name: str,
        owner_notification: Literal["send", "send-anon", "none"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        reg_who_request: abuse_report_create_params.AbuseReportsRegistrarWhoisReportRegWhoRequest | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reg_who_request: RDP-mandated fields for registrar WHOIS data disclosure requests.

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_ncsei"],
        email: str,
        email2: str,
        host_notification: Literal["send", "send-anon"],
        name: str,
        ncsei_subject_representation: bool,
        owner_notification: Literal["send", "send-anon", "none"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        country: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          ncsei_subject_representation: If the submitter is the target of NCSEI in the URLs of the abuse report.

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          country: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        [
            "account_id",
            "act",
            "address1",
            "agent_name",
            "agree",
            "city",
            "country",
            "email",
            "email2",
            "host_notification",
            "name",
            "original_work",
            "owner_notification",
            "signature",
            "state",
            "urls",
        ],
        [
            "account_id",
            "act",
            "email",
            "email2",
            "host_notification",
            "justification",
            "name",
            "owner_notification",
            "trademark_number",
            "trademark_office",
            "trademark_symbol",
            "urls",
        ],
        [
            "account_id",
            "act",
            "email",
            "email2",
            "host_notification",
            "justification",
            "name",
            "owner_notification",
            "urls",
        ],
        [
            "account_id",
            "act",
            "email",
            "email2",
            "host_notification",
            "justification",
            "name",
            "ncmec_notification",
            "owner_notification",
            "urls",
        ],
        ["account_id", "act", "email", "email2", "name", "owner_notification", "urls"],
        [
            "account_id",
            "act",
            "email",
            "email2",
            "host_notification",
            "name",
            "ncsei_subject_representation",
            "owner_notification",
            "urls",
        ],
    )
    def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_dmca"]
        | Literal["abuse_trademark"]
        | Literal["abuse_general"]
        | Literal["abuse_phishing"]
        | Literal["abuse_children"]
        | Literal["abuse_threat"]
        | Literal["abuse_registrar_whois"]
        | Literal["abuse_ncsei"],
        address1: str | Omit = omit,
        agent_name: str | Omit = omit,
        agree: Literal[1] | Omit = omit,
        city: str | Omit = omit,
        country: str | Omit = omit,
        email: str,
        email2: str,
        host_notification: Literal["send"] | Literal["send", "send-anon"] | Omit = omit,
        name: str,
        original_work: str | Omit = omit,
        owner_notification: Literal["send"] | Literal["send", "send-anon"] | Literal["send", "send-anon", "none"],
        signature: str | Omit = omit,
        state: str | Omit = omit,
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        justification: str | Omit = omit,
        trademark_number: str | Omit = omit,
        trademark_office: str | Omit = omit,
        trademark_symbol: str | Omit = omit,
        destination_ips: str | Omit = omit,
        ports_protocols: str | Omit = omit,
        source_ips: str | Omit = omit,
        ncmec_notification: Literal["send", "send-anon"] | Omit = omit,
        reg_who_request: abuse_report_create_params.AbuseReportsRegistrarWhoisReportRegWhoRequest | Omit = omit,
        ncsei_subject_representation: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not report_param:
            raise ValueError(f"Expected a non-empty value for `report_param` but received {report_param!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/abuse-reports/{report_param}", account_id=account_id, report_param=report_param
            ),
            body=maybe_transform(
                {
                    "act": act,
                    "address1": address1,
                    "agent_name": agent_name,
                    "agree": agree,
                    "city": city,
                    "country": country,
                    "email": email,
                    "email2": email2,
                    "host_notification": host_notification,
                    "name": name,
                    "original_work": original_work,
                    "owner_notification": owner_notification,
                    "signature": signature,
                    "state": state,
                    "urls": urls,
                    "comments": comments,
                    "company": company,
                    "reported_country": reported_country,
                    "reported_user_agent": reported_user_agent,
                    "tele": tele,
                    "title": title,
                    "justification": justification,
                    "trademark_number": trademark_number,
                    "trademark_office": trademark_office,
                    "trademark_symbol": trademark_symbol,
                    "destination_ips": destination_ips,
                    "ports_protocols": ports_protocols,
                    "source_ips": source_ips,
                    "ncmec_notification": ncmec_notification,
                    "reg_who_request": reg_who_request,
                    "ncsei_subject_representation": ncsei_subject_representation,
                },
                abuse_report_create_params.AbuseReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AbuseReportCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    def list(
        self,
        *,
        account_id: str,
        created_after: str | Omit = omit,
        created_before: str | Omit = omit,
        domain: str | Omit = omit,
        mitigation_status: Literal["pending", "active", "in_review", "cancelled", "removed"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        sort: str | Omit = omit,
        status: Literal["accepted", "in_review"] | Omit = omit,
        type: Literal["PHISH", "GEN", "THREAT", "DMCA", "EMER", "TM", "REG_WHO", "NCSEI", "NETWORK"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePagination[Optional[AbuseReportListResponse]]:
        """
        List the abuse reports for a given account

        Args:
          created_after: Returns reports created after the specified date

          created_before: Returns reports created before the specified date

          domain: Filter by domain name related to the abuse report

          mitigation_status: Filter reports that have any mitigations in the given status.

          page: Where in pagination to start listing abuse reports

          per_page: How many abuse reports per page to list

          sort: A property to sort by, followed by the order (id, cdate, domain, type, status)

          status: Filter by the status of the report.

          type: Filter by the type of the report.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/abuse-reports", account_id=account_id),
            page=SyncV4PagePagination[Optional[AbuseReportListResponse]],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_after": created_after,
                        "created_before": created_before,
                        "domain": domain,
                        "mitigation_status": mitigation_status,
                        "page": page,
                        "per_page": per_page,
                        "sort": sort,
                        "status": status,
                        "type": type,
                    },
                    abuse_report_list_params.AbuseReportListParams,
                ),
            ),
            model=AbuseReportListResponse,
        )

    def get(
        self,
        report_param: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AbuseReportGetResponse:
        """
        Retrieve the details of an abuse report.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not report_param:
            raise ValueError(f"Expected a non-empty value for `report_param` but received {report_param!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/abuse-reports/{report_param}", account_id=account_id, report_param=report_param
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AbuseReportGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AbuseReportGetResponse], ResultWrapper[AbuseReportGetResponse]),
        )


class AsyncAbuseReportsResource(AsyncAPIResource):
    @cached_property
    def mitigations(self) -> AsyncMitigationsResource:
        return AsyncMitigationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAbuseReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAbuseReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAbuseReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAbuseReportsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_dmca"],
        address1: str,
        agent_name: str,
        agree: Literal[1],
        city: str,
        country: str,
        email: str,
        email2: str,
        host_notification: Literal["send"],
        name: str,
        original_work: str,
        owner_notification: Literal["send"],
        signature: str,
        state: str,
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          address1: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          agent_name: The name of the copyright holder. Text not exceeding 60 characters. This field
              may be released by Cloudflare to third parties such as the Lumen Database
              (https://lumendatabase.org/).

          agree: Can be `0` for false or `1` for true. Must be value: 1 for DMCA reports

          city: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          country: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          original_work: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          signature: Required for DMCA reports, should be same as Name. An affirmation that all
              information in the report is true and accurate while agreeing to the policies of
              Cloudflare's abuse reports

          state: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_trademark"],
        email: str,
        email2: str,
        host_notification: Literal["send"],
        justification: str,
        name: str,
        owner_notification: Literal["send"],
        trademark_number: str,
        trademark_office: str,
        trademark_symbol: str,
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          justification: A detailed description of the infringement, including any necessary access
              details and the exact steps needed to view the content, not exceeding 5000
              characters.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          trademark_number: Text not exceeding 1000 characters

          trademark_office: Text not exceeding 1000 characters

          trademark_symbol: Text not exceeding 1000 characters

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_general"],
        email: str,
        email2: str,
        host_notification: Literal["send", "send-anon"],
        justification: str,
        name: str,
        owner_notification: Literal["send", "send-anon"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        destination_ips: str | Omit = omit,
        ports_protocols: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        source_ips: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          justification: A detailed description of the infringement, including any necessary access
              details and the exact steps needed to view the content, not exceeding 5000
              characters.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          destination_ips: A list of IP addresses separated by ‘\n’ (new line character). The list of
              destination IPs should not exceed 30 IP addresses. Each one of the IP addresses
              ought to be unique.

          ports_protocols: A comma separated list of ports and protocols e.g. 80/TCP, 22/UDP. The total
              size of the field should not exceed 2000 characters. Each individual
              port/protocol should not exceed 100 characters. The list should not have more
              than 30 unique ports and protocols.

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          source_ips: A list of IP addresses separated by ‘\n’ (new line character). The list of
              source IPs should not exceed 30 IP addresses. Each one of the IP addresses ought
              to be unique.

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_phishing"],
        email: str,
        email2: str,
        host_notification: Literal["send", "send-anon"],
        justification: str,
        name: str,
        owner_notification: Literal["send", "send-anon"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        original_work: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          justification: A detailed description of the infringement, including any necessary access
              details and the exact steps needed to view the content, not exceeding 5000
              characters.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          original_work: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_children"],
        email: str,
        email2: str,
        host_notification: Literal["send", "send-anon"],
        justification: str,
        name: str,
        ncmec_notification: Literal["send", "send-anon"],
        owner_notification: Literal["send", "send-anon", "none"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        country: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          justification: A detailed description of the infringement, including any necessary access
              details and the exact steps needed to view the content, not exceeding 5000
              characters.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          ncmec_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          country: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_threat"],
        email: str,
        email2: str,
        host_notification: Literal["send", "send-anon"],
        justification: str,
        name: str,
        owner_notification: Literal["send", "send-anon"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          justification: A detailed description of the infringement, including any necessary access
              details and the exact steps needed to view the content, not exceeding 5000
              characters.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_registrar_whois"],
        email: str,
        email2: str,
        name: str,
        owner_notification: Literal["send", "send-anon", "none"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        reg_who_request: abuse_report_create_params.AbuseReportsRegistrarWhoisReportRegWhoRequest | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reg_who_request: RDP-mandated fields for registrar WHOIS data disclosure requests.

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_ncsei"],
        email: str,
        email2: str,
        host_notification: Literal["send", "send-anon"],
        name: str,
        ncsei_subject_representation: bool,
        owner_notification: Literal["send", "send-anon", "none"],
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        country: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Submit the Abuse Report of a particular type

        Args:
          report_param: The report type for submitted reports.

          act: The report type for submitted reports.

          email: A valid email of the abuse reporter. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          email2: Should match the value provided in `email`

          host_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          name: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          ncsei_subject_representation: If the submitter is the target of NCSEI in the URLs of the abuse report.

          owner_notification: Notification type based on the abuse type. NOTE: Copyright (DMCA) and Trademark
              reports cannot be anonymous.

          urls: A list of valid URLs separated by ‘\n’ (new line character). The list of the
              URLs should not exceed 250 URLs. All URLs should have the same hostname. Each
              URL should be unique. This field may be released by Cloudflare to third parties
              such as the Lumen Database (https://lumendatabase.org/).

          comments: Any additional comments about the infringement not exceeding 2000 characters

          company: Text not exceeding 100 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          country: Text not exceeding 255 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          reported_country: Text containing 2 characters

          reported_user_agent: Text not exceeding 255 characters

          tele: Text not exceeding 20 characters. This field may be released by Cloudflare to
              third parties such as the Lumen Database (https://lumendatabase.org/).

          title: Text not exceeding 255 characters

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        [
            "account_id",
            "act",
            "address1",
            "agent_name",
            "agree",
            "city",
            "country",
            "email",
            "email2",
            "host_notification",
            "name",
            "original_work",
            "owner_notification",
            "signature",
            "state",
            "urls",
        ],
        [
            "account_id",
            "act",
            "email",
            "email2",
            "host_notification",
            "justification",
            "name",
            "owner_notification",
            "trademark_number",
            "trademark_office",
            "trademark_symbol",
            "urls",
        ],
        [
            "account_id",
            "act",
            "email",
            "email2",
            "host_notification",
            "justification",
            "name",
            "owner_notification",
            "urls",
        ],
        [
            "account_id",
            "act",
            "email",
            "email2",
            "host_notification",
            "justification",
            "name",
            "ncmec_notification",
            "owner_notification",
            "urls",
        ],
        ["account_id", "act", "email", "email2", "name", "owner_notification", "urls"],
        [
            "account_id",
            "act",
            "email",
            "email2",
            "host_notification",
            "name",
            "ncsei_subject_representation",
            "owner_notification",
            "urls",
        ],
    )
    async def create(
        self,
        report_param: str,
        *,
        account_id: str,
        act: Literal["abuse_dmca"]
        | Literal["abuse_trademark"]
        | Literal["abuse_general"]
        | Literal["abuse_phishing"]
        | Literal["abuse_children"]
        | Literal["abuse_threat"]
        | Literal["abuse_registrar_whois"]
        | Literal["abuse_ncsei"],
        address1: str | Omit = omit,
        agent_name: str | Omit = omit,
        agree: Literal[1] | Omit = omit,
        city: str | Omit = omit,
        country: str | Omit = omit,
        email: str,
        email2: str,
        host_notification: Literal["send"] | Literal["send", "send-anon"] | Omit = omit,
        name: str,
        original_work: str | Omit = omit,
        owner_notification: Literal["send"] | Literal["send", "send-anon"] | Literal["send", "send-anon", "none"],
        signature: str | Omit = omit,
        state: str | Omit = omit,
        urls: str,
        comments: str | Omit = omit,
        company: str | Omit = omit,
        reported_country: str | Omit = omit,
        reported_user_agent: str | Omit = omit,
        tele: str | Omit = omit,
        title: str | Omit = omit,
        justification: str | Omit = omit,
        trademark_number: str | Omit = omit,
        trademark_office: str | Omit = omit,
        trademark_symbol: str | Omit = omit,
        destination_ips: str | Omit = omit,
        ports_protocols: str | Omit = omit,
        source_ips: str | Omit = omit,
        ncmec_notification: Literal["send", "send-anon"] | Omit = omit,
        reg_who_request: abuse_report_create_params.AbuseReportsRegistrarWhoisReportRegWhoRequest | Omit = omit,
        ncsei_subject_representation: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not report_param:
            raise ValueError(f"Expected a non-empty value for `report_param` but received {report_param!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/abuse-reports/{report_param}", account_id=account_id, report_param=report_param
            ),
            body=await async_maybe_transform(
                {
                    "act": act,
                    "address1": address1,
                    "agent_name": agent_name,
                    "agree": agree,
                    "city": city,
                    "country": country,
                    "email": email,
                    "email2": email2,
                    "host_notification": host_notification,
                    "name": name,
                    "original_work": original_work,
                    "owner_notification": owner_notification,
                    "signature": signature,
                    "state": state,
                    "urls": urls,
                    "comments": comments,
                    "company": company,
                    "reported_country": reported_country,
                    "reported_user_agent": reported_user_agent,
                    "tele": tele,
                    "title": title,
                    "justification": justification,
                    "trademark_number": trademark_number,
                    "trademark_office": trademark_office,
                    "trademark_symbol": trademark_symbol,
                    "destination_ips": destination_ips,
                    "ports_protocols": ports_protocols,
                    "source_ips": source_ips,
                    "ncmec_notification": ncmec_notification,
                    "reg_who_request": reg_who_request,
                    "ncsei_subject_representation": ncsei_subject_representation,
                },
                abuse_report_create_params.AbuseReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AbuseReportCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    def list(
        self,
        *,
        account_id: str,
        created_after: str | Omit = omit,
        created_before: str | Omit = omit,
        domain: str | Omit = omit,
        mitigation_status: Literal["pending", "active", "in_review", "cancelled", "removed"] | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        sort: str | Omit = omit,
        status: Literal["accepted", "in_review"] | Omit = omit,
        type: Literal["PHISH", "GEN", "THREAT", "DMCA", "EMER", "TM", "REG_WHO", "NCSEI", "NETWORK"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Optional[AbuseReportListResponse], AsyncV4PagePagination[Optional[AbuseReportListResponse]]]:
        """
        List the abuse reports for a given account

        Args:
          created_after: Returns reports created after the specified date

          created_before: Returns reports created before the specified date

          domain: Filter by domain name related to the abuse report

          mitigation_status: Filter reports that have any mitigations in the given status.

          page: Where in pagination to start listing abuse reports

          per_page: How many abuse reports per page to list

          sort: A property to sort by, followed by the order (id, cdate, domain, type, status)

          status: Filter by the status of the report.

          type: Filter by the type of the report.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/abuse-reports", account_id=account_id),
            page=AsyncV4PagePagination[Optional[AbuseReportListResponse]],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "created_after": created_after,
                        "created_before": created_before,
                        "domain": domain,
                        "mitigation_status": mitigation_status,
                        "page": page,
                        "per_page": per_page,
                        "sort": sort,
                        "status": status,
                        "type": type,
                    },
                    abuse_report_list_params.AbuseReportListParams,
                ),
            ),
            model=AbuseReportListResponse,
        )

    async def get(
        self,
        report_param: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AbuseReportGetResponse:
        """
        Retrieve the details of an abuse report.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not report_param:
            raise ValueError(f"Expected a non-empty value for `report_param` but received {report_param!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/abuse-reports/{report_param}", account_id=account_id, report_param=report_param
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AbuseReportGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AbuseReportGetResponse], ResultWrapper[AbuseReportGetResponse]),
        )


class AbuseReportsResourceWithRawResponse:
    def __init__(self, abuse_reports: AbuseReportsResource) -> None:
        self._abuse_reports = abuse_reports

        self.create = to_raw_response_wrapper(
            abuse_reports.create,
        )
        self.list = to_raw_response_wrapper(
            abuse_reports.list,
        )
        self.get = to_raw_response_wrapper(
            abuse_reports.get,
        )

    @cached_property
    def mitigations(self) -> MitigationsResourceWithRawResponse:
        return MitigationsResourceWithRawResponse(self._abuse_reports.mitigations)


class AsyncAbuseReportsResourceWithRawResponse:
    def __init__(self, abuse_reports: AsyncAbuseReportsResource) -> None:
        self._abuse_reports = abuse_reports

        self.create = async_to_raw_response_wrapper(
            abuse_reports.create,
        )
        self.list = async_to_raw_response_wrapper(
            abuse_reports.list,
        )
        self.get = async_to_raw_response_wrapper(
            abuse_reports.get,
        )

    @cached_property
    def mitigations(self) -> AsyncMitigationsResourceWithRawResponse:
        return AsyncMitigationsResourceWithRawResponse(self._abuse_reports.mitigations)


class AbuseReportsResourceWithStreamingResponse:
    def __init__(self, abuse_reports: AbuseReportsResource) -> None:
        self._abuse_reports = abuse_reports

        self.create = to_streamed_response_wrapper(
            abuse_reports.create,
        )
        self.list = to_streamed_response_wrapper(
            abuse_reports.list,
        )
        self.get = to_streamed_response_wrapper(
            abuse_reports.get,
        )

    @cached_property
    def mitigations(self) -> MitigationsResourceWithStreamingResponse:
        return MitigationsResourceWithStreamingResponse(self._abuse_reports.mitigations)


class AsyncAbuseReportsResourceWithStreamingResponse:
    def __init__(self, abuse_reports: AsyncAbuseReportsResource) -> None:
        self._abuse_reports = abuse_reports

        self.create = async_to_streamed_response_wrapper(
            abuse_reports.create,
        )
        self.list = async_to_streamed_response_wrapper(
            abuse_reports.list,
        )
        self.get = async_to_streamed_response_wrapper(
            abuse_reports.get,
        )

    @cached_property
    def mitigations(self) -> AsyncMitigationsResourceWithStreamingResponse:
        return AsyncMitigationsResourceWithStreamingResponse(self._abuse_reports.mitigations)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypeAlias

__all__ = ["SortDirection"]

SortDirection: TypeAlias = Literal["asc", "desc"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .schema_get_params import SchemaGetParams as SchemaGetParams
from .schema_get_response import SchemaGetResponse as SchemaGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.bot_management import bot_management_update_params
from ...types.bot_management.bot_management_get_response import BotManagementGetResponse
from ...types.bot_management.bot_management_update_response import BotManagementUpdateResponse

__all__ = ["BotManagementResource", "AsyncBotManagementResource"]


class BotManagementResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BotManagementResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return BotManagementResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BotManagementResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return BotManagementResourceWithStreamingResponse(self)

    @overload
    def update(
        self,
        *,
        zone_id: str,
        ai_bots_protection: Literal["block", "disabled", "only_on_ad_pages"] | Omit = omit,
        cf_robots_variant: Literal["off", "policy_only"] | Omit = omit,
        content_bots_protection: Literal["block", "disabled"] | Omit = omit,
        crawler_protection: Literal["enabled", "disabled"] | Omit = omit,
        enable_js: bool | Omit = omit,
        fight_mode: bool | Omit = omit,
        is_robots_txt_managed: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementUpdateResponse]:
        """
        Updates the Bot Management configuration for a zone.

        This API is used to update:

        - **Bot Fight Mode**
        - **Super Bot Fight Mode**
        - **Bot Management for Enterprise**

        See [Bot Plans](https://developers.cloudflare.com/bots/plans/) for more
        information on the different plans \\
        If you recently upgraded or downgraded your plan, refer to the following examples
        to clean up old configurations. Copy and paste the example body to remove old zone
        configurations based on your current plan.

        #### Clean up configuration for Bot Fight Mode plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "suppress_session_score": false
        }
        ```

        #### Clean up configuration for SBFM Pro plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "fight_mode": false
        }
        ```

        #### Clean up configuration for SBFM Biz plan

        ```json
        {
          "fight_mode": false
        }
        ```

        #### Clean up configuration for BM Enterprise Subscription plan

        It is strongly recommended that you ensure you have
        [custom rules](https://developers.cloudflare.com/waf/custom-rules/) in place to
        protect your zone before disabling the SBFM rules. Without these protections,
        your zone is vulnerable to attacks.

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "fight_mode": false
        }
        ```

        Args:
          zone_id: Identifier.

          ai_bots_protection: Enable rule to block AI Scrapers and Crawlers. Please note the value
              `only_on_ad_pages` is currently not available for Enterprise customers.

          cf_robots_variant: Specifies the Robots Access Control License variant to use.

          content_bots_protection: Enable rule to block content bots. When enabled, blocks automated traffic with
              low bot scores, excluding safe verified bot categories. Exceptions should be
              managed via skip rules.

          crawler_protection: Enable rule to punish AI Scrapers and Crawlers via a link maze.

          enable_js: Use lightweight, invisible JavaScript detections to improve Bot Management.
              [Learn more about JavaScript Detections](https://developers.cloudflare.com/bots/reference/javascript-detections/).

          fight_mode: Whether to enable Bot Fight Mode.

          is_robots_txt_managed: Enable cloudflare managed robots.txt. If an existing robots.txt is detected,
              then managed robots.txt will be prepended to the existing robots.txt.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        *,
        zone_id: str,
        ai_bots_protection: Literal["block", "disabled", "only_on_ad_pages"] | Omit = omit,
        cf_robots_variant: Literal["off", "policy_only"] | Omit = omit,
        content_bots_protection: Literal["block", "disabled"] | Omit = omit,
        crawler_protection: Literal["enabled", "disabled"] | Omit = omit,
        enable_js: bool | Omit = omit,
        is_robots_txt_managed: bool | Omit = omit,
        optimize_wordpress: bool | Omit = omit,
        sbfm_definitely_automated: Literal["allow", "block", "managed_challenge"] | Omit = omit,
        sbfm_static_resource_protection: bool | Omit = omit,
        sbfm_verified_bots: Literal["allow", "block"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementUpdateResponse]:
        """
        Updates the Bot Management configuration for a zone.

        This API is used to update:

        - **Bot Fight Mode**
        - **Super Bot Fight Mode**
        - **Bot Management for Enterprise**

        See [Bot Plans](https://developers.cloudflare.com/bots/plans/) for more
        information on the different plans \\
        If you recently upgraded or downgraded your plan, refer to the following examples
        to clean up old configurations. Copy and paste the example body to remove old zone
        configurations based on your current plan.

        #### Clean up configuration for Bot Fight Mode plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "suppress_session_score": false
        }
        ```

        #### Clean up configuration for SBFM Pro plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "fight_mode": false
        }
        ```

        #### Clean up configuration for SBFM Biz plan

        ```json
        {
          "fight_mode": false
        }
        ```

        #### Clean up configuration for BM Enterprise Subscription plan

        It is strongly recommended that you ensure you have
        [custom rules](https://developers.cloudflare.com/waf/custom-rules/) in place to
        protect your zone before disabling the SBFM rules. Without these protections,
        your zone is vulnerable to attacks.

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "fight_mode": false
        }
        ```

        Args:
          zone_id: Identifier.

          ai_bots_protection: Enable rule to block AI Scrapers and Crawlers. Please note the value
              `only_on_ad_pages` is currently not available for Enterprise customers.

          cf_robots_variant: Specifies the Robots Access Control License variant to use.

          content_bots_protection: Enable rule to block content bots. When enabled, blocks automated traffic with
              low bot scores, excluding safe verified bot categories. Exceptions should be
              managed via skip rules.

          crawler_protection: Enable rule to punish AI Scrapers and Crawlers via a link maze.

          enable_js: Use lightweight, invisible JavaScript detections to improve Bot Management.
              [Learn more about JavaScript Detections](https://developers.cloudflare.com/bots/reference/javascript-detections/).

          is_robots_txt_managed: Enable cloudflare managed robots.txt. If an existing robots.txt is detected,
              then managed robots.txt will be prepended to the existing robots.txt.

          optimize_wordpress: Whether to optimize Super Bot Fight Mode protections for Wordpress.

          sbfm_definitely_automated: Super Bot Fight Mode (SBFM) action to take on definitely automated requests.

          sbfm_static_resource_protection: Super Bot Fight Mode (SBFM) to enable static resource protection. Enable if
              static resources on your application need bot protection. Note: Static resource
              protection can also result in legitimate traffic being blocked.

          sbfm_verified_bots: Super Bot Fight Mode (SBFM) action to take on verified bots requests.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        *,
        zone_id: str,
        ai_bots_protection: Literal["block", "disabled", "only_on_ad_pages"] | Omit = omit,
        cf_robots_variant: Literal["off", "policy_only"] | Omit = omit,
        content_bots_protection: Literal["block", "disabled"] | Omit = omit,
        crawler_protection: Literal["enabled", "disabled"] | Omit = omit,
        enable_js: bool | Omit = omit,
        is_robots_txt_managed: bool | Omit = omit,
        optimize_wordpress: bool | Omit = omit,
        sbfm_definitely_automated: Literal["allow", "block", "managed_challenge"] | Omit = omit,
        sbfm_likely_automated: Literal["allow", "block", "managed_challenge"] | Omit = omit,
        sbfm_static_resource_protection: bool | Omit = omit,
        sbfm_verified_bots: Literal["allow", "block"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementUpdateResponse]:
        """
        Updates the Bot Management configuration for a zone.

        This API is used to update:

        - **Bot Fight Mode**
        - **Super Bot Fight Mode**
        - **Bot Management for Enterprise**

        See [Bot Plans](https://developers.cloudflare.com/bots/plans/) for more
        information on the different plans \\
        If you recently upgraded or downgraded your plan, refer to the following examples
        to clean up old configurations. Copy and paste the example body to remove old zone
        configurations based on your current plan.

        #### Clean up configuration for Bot Fight Mode plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "suppress_session_score": false
        }
        ```

        #### Clean up configuration for SBFM Pro plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "fight_mode": false
        }
        ```

        #### Clean up configuration for SBFM Biz plan

        ```json
        {
          "fight_mode": false
        }
        ```

        #### Clean up configuration for BM Enterprise Subscription plan

        It is strongly recommended that you ensure you have
        [custom rules](https://developers.cloudflare.com/waf/custom-rules/) in place to
        protect your zone before disabling the SBFM rules. Without these protections,
        your zone is vulnerable to attacks.

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "fight_mode": false
        }
        ```

        Args:
          zone_id: Identifier.

          ai_bots_protection: Enable rule to block AI Scrapers and Crawlers. Please note the value
              `only_on_ad_pages` is currently not available for Enterprise customers.

          cf_robots_variant: Specifies the Robots Access Control License variant to use.

          content_bots_protection: Enable rule to block content bots. When enabled, blocks automated traffic with
              low bot scores, excluding safe verified bot categories. Exceptions should be
              managed via skip rules.

          crawler_protection: Enable rule to punish AI Scrapers and Crawlers via a link maze.

          enable_js: Use lightweight, invisible JavaScript detections to improve Bot Management.
              [Learn more about JavaScript Detections](https://developers.cloudflare.com/bots/reference/javascript-detections/).

          is_robots_txt_managed: Enable cloudflare managed robots.txt. If an existing robots.txt is detected,
              then managed robots.txt will be prepended to the existing robots.txt.

          optimize_wordpress: Whether to optimize Super Bot Fight Mode protections for Wordpress.

          sbfm_definitely_automated: Super Bot Fight Mode (SBFM) action to take on definitely automated requests.

          sbfm_likely_automated: Super Bot Fight Mode (SBFM) action to take on likely automated requests.

          sbfm_static_resource_protection: Super Bot Fight Mode (SBFM) to enable static resource protection. Enable if
              static resources on your application need bot protection. Note: Static resource
              protection can also result in legitimate traffic being blocked.

          sbfm_verified_bots: Super Bot Fight Mode (SBFM) action to take on verified bots requests.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        *,
        zone_id: str,
        ai_bots_protection: Literal["block", "disabled", "only_on_ad_pages"] | Omit = omit,
        auto_update_model: bool | Omit = omit,
        bm_cookie_enabled: bool | Omit = omit,
        cf_robots_variant: Literal["off", "policy_only"] | Omit = omit,
        content_bots_protection: Literal["block", "disabled"] | Omit = omit,
        crawler_protection: Literal["enabled", "disabled"] | Omit = omit,
        enable_js: bool | Omit = omit,
        is_robots_txt_managed: bool | Omit = omit,
        suppress_session_score: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementUpdateResponse]:
        """
        Updates the Bot Management configuration for a zone.

        This API is used to update:

        - **Bot Fight Mode**
        - **Super Bot Fight Mode**
        - **Bot Management for Enterprise**

        See [Bot Plans](https://developers.cloudflare.com/bots/plans/) for more
        information on the different plans \\
        If you recently upgraded or downgraded your plan, refer to the following examples
        to clean up old configurations. Copy and paste the example body to remove old zone
        configurations based on your current plan.

        #### Clean up configuration for Bot Fight Mode plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "suppress_session_score": false
        }
        ```

        #### Clean up configuration for SBFM Pro plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "fight_mode": false
        }
        ```

        #### Clean up configuration for SBFM Biz plan

        ```json
        {
          "fight_mode": false
        }
        ```

        #### Clean up configuration for BM Enterprise Subscription plan

        It is strongly recommended that you ensure you have
        [custom rules](https://developers.cloudflare.com/waf/custom-rules/) in place to
        protect your zone before disabling the SBFM rules. Without these protections,
        your zone is vulnerable to attacks.

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "fight_mode": false
        }
        ```

        Args:
          zone_id: Identifier.

          ai_bots_protection: Enable rule to block AI Scrapers and Crawlers. Please note the value
              `only_on_ad_pages` is currently not available for Enterprise customers.

          auto_update_model: Automatically update to the newest bot detection models created by Cloudflare as
              they are released.
              [Learn more.](https://developers.cloudflare.com/bots/reference/machine-learning-models#model-versions-and-release-notes)

          bm_cookie_enabled: Indicates that the bot management cookie can be placed on end user devices
              accessing the site. Defaults to true

          cf_robots_variant: Specifies the Robots Access Control License variant to use.

          content_bots_protection: Enable rule to block content bots. When enabled, blocks automated traffic with
              low bot scores, excluding safe verified bot categories. Exceptions should be
              managed via skip rules.

          crawler_protection: Enable rule to punish AI Scrapers and Crawlers via a link maze.

          enable_js: Use lightweight, invisible JavaScript detections to improve Bot Management.
              [Learn more about JavaScript Detections](https://developers.cloudflare.com/bots/reference/javascript-detections/).

          is_robots_txt_managed: Enable cloudflare managed robots.txt. If an existing robots.txt is detected,
              then managed robots.txt will be prepended to the existing robots.txt.

          suppress_session_score: Whether to disable tracking the highest bot score for a session in the Bot
              Management cookie.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["zone_id"])
    def update(
        self,
        *,
        zone_id: str,
        ai_bots_protection: Literal["block", "disabled", "only_on_ad_pages"] | Omit = omit,
        cf_robots_variant: Literal["off", "policy_only"] | Omit = omit,
        content_bots_protection: Literal["block", "disabled"] | Omit = omit,
        crawler_protection: Literal["enabled", "disabled"] | Omit = omit,
        enable_js: bool | Omit = omit,
        fight_mode: bool | Omit = omit,
        is_robots_txt_managed: bool | Omit = omit,
        optimize_wordpress: bool | Omit = omit,
        sbfm_definitely_automated: Literal["allow", "block", "managed_challenge"] | Omit = omit,
        sbfm_static_resource_protection: bool | Omit = omit,
        sbfm_verified_bots: Literal["allow", "block"] | Omit = omit,
        sbfm_likely_automated: Literal["allow", "block", "managed_challenge"] | Omit = omit,
        auto_update_model: bool | Omit = omit,
        bm_cookie_enabled: bool | Omit = omit,
        suppress_session_score: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementUpdateResponse]:
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            Optional[BotManagementUpdateResponse],
            self._put(
                path_template("/zones/{zone_id}/bot_management", zone_id=zone_id),
                body=maybe_transform(
                    {
                        "ai_bots_protection": ai_bots_protection,
                        "cf_robots_variant": cf_robots_variant,
                        "content_bots_protection": content_bots_protection,
                        "crawler_protection": crawler_protection,
                        "enable_js": enable_js,
                        "fight_mode": fight_mode,
                        "is_robots_txt_managed": is_robots_txt_managed,
                        "optimize_wordpress": optimize_wordpress,
                        "sbfm_definitely_automated": sbfm_definitely_automated,
                        "sbfm_static_resource_protection": sbfm_static_resource_protection,
                        "sbfm_verified_bots": sbfm_verified_bots,
                        "sbfm_likely_automated": sbfm_likely_automated,
                        "auto_update_model": auto_update_model,
                        "bm_cookie_enabled": bm_cookie_enabled,
                        "suppress_session_score": suppress_session_score,
                    },
                    bot_management_update_params.BotManagementUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[BotManagementUpdateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[BotManagementUpdateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementGetResponse]:
        """
        Retrieve a zone's Bot Management Config

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            Optional[BotManagementGetResponse],
            self._get(
                path_template("/zones/{zone_id}/bot_management", zone_id=zone_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[BotManagementGetResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[BotManagementGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncBotManagementResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBotManagementResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBotManagementResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBotManagementResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncBotManagementResourceWithStreamingResponse(self)

    @overload
    async def update(
        self,
        *,
        zone_id: str,
        ai_bots_protection: Literal["block", "disabled", "only_on_ad_pages"] | Omit = omit,
        cf_robots_variant: Literal["off", "policy_only"] | Omit = omit,
        content_bots_protection: Literal["block", "disabled"] | Omit = omit,
        crawler_protection: Literal["enabled", "disabled"] | Omit = omit,
        enable_js: bool | Omit = omit,
        fight_mode: bool | Omit = omit,
        is_robots_txt_managed: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementUpdateResponse]:
        """
        Updates the Bot Management configuration for a zone.

        This API is used to update:

        - **Bot Fight Mode**
        - **Super Bot Fight Mode**
        - **Bot Management for Enterprise**

        See [Bot Plans](https://developers.cloudflare.com/bots/plans/) for more
        information on the different plans \\
        If you recently upgraded or downgraded your plan, refer to the following examples
        to clean up old configurations. Copy and paste the example body to remove old zone
        configurations based on your current plan.

        #### Clean up configuration for Bot Fight Mode plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "suppress_session_score": false
        }
        ```

        #### Clean up configuration for SBFM Pro plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "fight_mode": false
        }
        ```

        #### Clean up configuration for SBFM Biz plan

        ```json
        {
          "fight_mode": false
        }
        ```

        #### Clean up configuration for BM Enterprise Subscription plan

        It is strongly recommended that you ensure you have
        [custom rules](https://developers.cloudflare.com/waf/custom-rules/) in place to
        protect your zone before disabling the SBFM rules. Without these protections,
        your zone is vulnerable to attacks.

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "fight_mode": false
        }
        ```

        Args:
          zone_id: Identifier.

          ai_bots_protection: Enable rule to block AI Scrapers and Crawlers. Please note the value
              `only_on_ad_pages` is currently not available for Enterprise customers.

          cf_robots_variant: Specifies the Robots Access Control License variant to use.

          content_bots_protection: Enable rule to block content bots. When enabled, blocks automated traffic with
              low bot scores, excluding safe verified bot categories. Exceptions should be
              managed via skip rules.

          crawler_protection: Enable rule to punish AI Scrapers and Crawlers via a link maze.

          enable_js: Use lightweight, invisible JavaScript detections to improve Bot Management.
              [Learn more about JavaScript Detections](https://developers.cloudflare.com/bots/reference/javascript-detections/).

          fight_mode: Whether to enable Bot Fight Mode.

          is_robots_txt_managed: Enable cloudflare managed robots.txt. If an existing robots.txt is detected,
              then managed robots.txt will be prepended to the existing robots.txt.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        *,
        zone_id: str,
        ai_bots_protection: Literal["block", "disabled", "only_on_ad_pages"] | Omit = omit,
        cf_robots_variant: Literal["off", "policy_only"] | Omit = omit,
        content_bots_protection: Literal["block", "disabled"] | Omit = omit,
        crawler_protection: Literal["enabled", "disabled"] | Omit = omit,
        enable_js: bool | Omit = omit,
        is_robots_txt_managed: bool | Omit = omit,
        optimize_wordpress: bool | Omit = omit,
        sbfm_definitely_automated: Literal["allow", "block", "managed_challenge"] | Omit = omit,
        sbfm_static_resource_protection: bool | Omit = omit,
        sbfm_verified_bots: Literal["allow", "block"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementUpdateResponse]:
        """
        Updates the Bot Management configuration for a zone.

        This API is used to update:

        - **Bot Fight Mode**
        - **Super Bot Fight Mode**
        - **Bot Management for Enterprise**

        See [Bot Plans](https://developers.cloudflare.com/bots/plans/) for more
        information on the different plans \\
        If you recently upgraded or downgraded your plan, refer to the following examples
        to clean up old configurations. Copy and paste the example body to remove old zone
        configurations based on your current plan.

        #### Clean up configuration for Bot Fight Mode plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "suppress_session_score": false
        }
        ```

        #### Clean up configuration for SBFM Pro plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "fight_mode": false
        }
        ```

        #### Clean up configuration for SBFM Biz plan

        ```json
        {
          "fight_mode": false
        }
        ```

        #### Clean up configuration for BM Enterprise Subscription plan

        It is strongly recommended that you ensure you have
        [custom rules](https://developers.cloudflare.com/waf/custom-rules/) in place to
        protect your zone before disabling the SBFM rules. Without these protections,
        your zone is vulnerable to attacks.

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "fight_mode": false
        }
        ```

        Args:
          zone_id: Identifier.

          ai_bots_protection: Enable rule to block AI Scrapers and Crawlers. Please note the value
              `only_on_ad_pages` is currently not available for Enterprise customers.

          cf_robots_variant: Specifies the Robots Access Control License variant to use.

          content_bots_protection: Enable rule to block content bots. When enabled, blocks automated traffic with
              low bot scores, excluding safe verified bot categories. Exceptions should be
              managed via skip rules.

          crawler_protection: Enable rule to punish AI Scrapers and Crawlers via a link maze.

          enable_js: Use lightweight, invisible JavaScript detections to improve Bot Management.
              [Learn more about JavaScript Detections](https://developers.cloudflare.com/bots/reference/javascript-detections/).

          is_robots_txt_managed: Enable cloudflare managed robots.txt. If an existing robots.txt is detected,
              then managed robots.txt will be prepended to the existing robots.txt.

          optimize_wordpress: Whether to optimize Super Bot Fight Mode protections for Wordpress.

          sbfm_definitely_automated: Super Bot Fight Mode (SBFM) action to take on definitely automated requests.

          sbfm_static_resource_protection: Super Bot Fight Mode (SBFM) to enable static resource protection. Enable if
              static resources on your application need bot protection. Note: Static resource
              protection can also result in legitimate traffic being blocked.

          sbfm_verified_bots: Super Bot Fight Mode (SBFM) action to take on verified bots requests.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        *,
        zone_id: str,
        ai_bots_protection: Literal["block", "disabled", "only_on_ad_pages"] | Omit = omit,
        cf_robots_variant: Literal["off", "policy_only"] | Omit = omit,
        content_bots_protection: Literal["block", "disabled"] | Omit = omit,
        crawler_protection: Literal["enabled", "disabled"] | Omit = omit,
        enable_js: bool | Omit = omit,
        is_robots_txt_managed: bool | Omit = omit,
        optimize_wordpress: bool | Omit = omit,
        sbfm_definitely_automated: Literal["allow", "block", "managed_challenge"] | Omit = omit,
        sbfm_likely_automated: Literal["allow", "block", "managed_challenge"] | Omit = omit,
        sbfm_static_resource_protection: bool | Omit = omit,
        sbfm_verified_bots: Literal["allow", "block"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementUpdateResponse]:
        """
        Updates the Bot Management configuration for a zone.

        This API is used to update:

        - **Bot Fight Mode**
        - **Super Bot Fight Mode**
        - **Bot Management for Enterprise**

        See [Bot Plans](https://developers.cloudflare.com/bots/plans/) for more
        information on the different plans \\
        If you recently upgraded or downgraded your plan, refer to the following examples
        to clean up old configurations. Copy and paste the example body to remove old zone
        configurations based on your current plan.

        #### Clean up configuration for Bot Fight Mode plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "suppress_session_score": false
        }
        ```

        #### Clean up configuration for SBFM Pro plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "fight_mode": false
        }
        ```

        #### Clean up configuration for SBFM Biz plan

        ```json
        {
          "fight_mode": false
        }
        ```

        #### Clean up configuration for BM Enterprise Subscription plan

        It is strongly recommended that you ensure you have
        [custom rules](https://developers.cloudflare.com/waf/custom-rules/) in place to
        protect your zone before disabling the SBFM rules. Without these protections,
        your zone is vulnerable to attacks.

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "fight_mode": false
        }
        ```

        Args:
          zone_id: Identifier.

          ai_bots_protection: Enable rule to block AI Scrapers and Crawlers. Please note the value
              `only_on_ad_pages` is currently not available for Enterprise customers.

          cf_robots_variant: Specifies the Robots Access Control License variant to use.

          content_bots_protection: Enable rule to block content bots. When enabled, blocks automated traffic with
              low bot scores, excluding safe verified bot categories. Exceptions should be
              managed via skip rules.

          crawler_protection: Enable rule to punish AI Scrapers and Crawlers via a link maze.

          enable_js: Use lightweight, invisible JavaScript detections to improve Bot Management.
              [Learn more about JavaScript Detections](https://developers.cloudflare.com/bots/reference/javascript-detections/).

          is_robots_txt_managed: Enable cloudflare managed robots.txt. If an existing robots.txt is detected,
              then managed robots.txt will be prepended to the existing robots.txt.

          optimize_wordpress: Whether to optimize Super Bot Fight Mode protections for Wordpress.

          sbfm_definitely_automated: Super Bot Fight Mode (SBFM) action to take on definitely automated requests.

          sbfm_likely_automated: Super Bot Fight Mode (SBFM) action to take on likely automated requests.

          sbfm_static_resource_protection: Super Bot Fight Mode (SBFM) to enable static resource protection. Enable if
              static resources on your application need bot protection. Note: Static resource
              protection can also result in legitimate traffic being blocked.

          sbfm_verified_bots: Super Bot Fight Mode (SBFM) action to take on verified bots requests.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        *,
        zone_id: str,
        ai_bots_protection: Literal["block", "disabled", "only_on_ad_pages"] | Omit = omit,
        auto_update_model: bool | Omit = omit,
        bm_cookie_enabled: bool | Omit = omit,
        cf_robots_variant: Literal["off", "policy_only"] | Omit = omit,
        content_bots_protection: Literal["block", "disabled"] | Omit = omit,
        crawler_protection: Literal["enabled", "disabled"] | Omit = omit,
        enable_js: bool | Omit = omit,
        is_robots_txt_managed: bool | Omit = omit,
        suppress_session_score: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementUpdateResponse]:
        """
        Updates the Bot Management configuration for a zone.

        This API is used to update:

        - **Bot Fight Mode**
        - **Super Bot Fight Mode**
        - **Bot Management for Enterprise**

        See [Bot Plans](https://developers.cloudflare.com/bots/plans/) for more
        information on the different plans \\
        If you recently upgraded or downgraded your plan, refer to the following examples
        to clean up old configurations. Copy and paste the example body to remove old zone
        configurations based on your current plan.

        #### Clean up configuration for Bot Fight Mode plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "suppress_session_score": false
        }
        ```

        #### Clean up configuration for SBFM Pro plan

        ```json
        {
          "sbfm_likely_automated": "allow",
          "fight_mode": false
        }
        ```

        #### Clean up configuration for SBFM Biz plan

        ```json
        {
          "fight_mode": false
        }
        ```

        #### Clean up configuration for BM Enterprise Subscription plan

        It is strongly recommended that you ensure you have
        [custom rules](https://developers.cloudflare.com/waf/custom-rules/) in place to
        protect your zone before disabling the SBFM rules. Without these protections,
        your zone is vulnerable to attacks.

        ```json
        {
          "sbfm_likely_automated": "allow",
          "sbfm_definitely_automated": "allow",
          "sbfm_verified_bots": "allow",
          "sbfm_static_resource_protection": false,
          "optimize_wordpress": false,
          "fight_mode": false
        }
        ```

        Args:
          zone_id: Identifier.

          ai_bots_protection: Enable rule to block AI Scrapers and Crawlers. Please note the value
              `only_on_ad_pages` is currently not available for Enterprise customers.

          auto_update_model: Automatically update to the newest bot detection models created by Cloudflare as
              they are released.
              [Learn more.](https://developers.cloudflare.com/bots/reference/machine-learning-models#model-versions-and-release-notes)

          bm_cookie_enabled: Indicates that the bot management cookie can be placed on end user devices
              accessing the site. Defaults to true

          cf_robots_variant: Specifies the Robots Access Control License variant to use.

          content_bots_protection: Enable rule to block content bots. When enabled, blocks automated traffic with
              low bot scores, excluding safe verified bot categories. Exceptions should be
              managed via skip rules.

          crawler_protection: Enable rule to punish AI Scrapers and Crawlers via a link maze.

          enable_js: Use lightweight, invisible JavaScript detections to improve Bot Management.
              [Learn more about JavaScript Detections](https://developers.cloudflare.com/bots/reference/javascript-detections/).

          is_robots_txt_managed: Enable cloudflare managed robots.txt. If an existing robots.txt is detected,
              then managed robots.txt will be prepended to the existing robots.txt.

          suppress_session_score: Whether to disable tracking the highest bot score for a session in the Bot
              Management cookie.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["zone_id"])
    async def update(
        self,
        *,
        zone_id: str,
        ai_bots_protection: Literal["block", "disabled", "only_on_ad_pages"] | Omit = omit,
        cf_robots_variant: Literal["off", "policy_only"] | Omit = omit,
        content_bots_protection: Literal["block", "disabled"] | Omit = omit,
        crawler_protection: Literal["enabled", "disabled"] | Omit = omit,
        enable_js: bool | Omit = omit,
        fight_mode: bool | Omit = omit,
        is_robots_txt_managed: bool | Omit = omit,
        optimize_wordpress: bool | Omit = omit,
        sbfm_definitely_automated: Literal["allow", "block", "managed_challenge"] | Omit = omit,
        sbfm_static_resource_protection: bool | Omit = omit,
        sbfm_verified_bots: Literal["allow", "block"] | Omit = omit,
        sbfm_likely_automated: Literal["allow", "block", "managed_challenge"] | Omit = omit,
        auto_update_model: bool | Omit = omit,
        bm_cookie_enabled: bool | Omit = omit,
        suppress_session_score: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementUpdateResponse]:
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            Optional[BotManagementUpdateResponse],
            await self._put(
                path_template("/zones/{zone_id}/bot_management", zone_id=zone_id),
                body=await async_maybe_transform(
                    {
                        "ai_bots_protection": ai_bots_protection,
                        "cf_robots_variant": cf_robots_variant,
                        "content_bots_protection": content_bots_protection,
                        "crawler_protection": crawler_protection,
                        "enable_js": enable_js,
                        "fight_mode": fight_mode,
                        "is_robots_txt_managed": is_robots_txt_managed,
                        "optimize_wordpress": optimize_wordpress,
                        "sbfm_definitely_automated": sbfm_definitely_automated,
                        "sbfm_static_resource_protection": sbfm_static_resource_protection,
                        "sbfm_verified_bots": sbfm_verified_bots,
                        "sbfm_likely_automated": sbfm_likely_automated,
                        "auto_update_model": auto_update_model,
                        "bm_cookie_enabled": bm_cookie_enabled,
                        "suppress_session_score": suppress_session_score,
                    },
                    bot_management_update_params.BotManagementUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[BotManagementUpdateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[BotManagementUpdateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[BotManagementGetResponse]:
        """
        Retrieve a zone's Bot Management Config

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            Optional[BotManagementGetResponse],
            await self._get(
                path_template("/zones/{zone_id}/bot_management", zone_id=zone_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[BotManagementGetResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[BotManagementGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class BotManagementResourceWithRawResponse:
    def __init__(self, bot_management: BotManagementResource) -> None:
        self._bot_management = bot_management

        self.update = to_raw_response_wrapper(
            bot_management.update,
        )
        self.get = to_raw_response_wrapper(
            bot_management.get,
        )


class AsyncBotManagementResourceWithRawResponse:
    def __init__(self, bot_management: AsyncBotManagementResource) -> None:
        self._bot_management = bot_management

        self.update = async_to_raw_response_wrapper(
            bot_management.update,
        )
        self.get = async_to_raw_response_wrapper(
            bot_management.get,
        )


class BotManagementResourceWithStreamingResponse:
    def __init__(self, bot_management: BotManagementResource) -> None:
        self._bot_management = bot_management

        self.update = to_streamed_response_wrapper(
            bot_management.update,
        )
        self.get = to_streamed_response_wrapper(
            bot_management.get,
        )


class AsyncBotManagementResourceWithStreamingResponse:
    def __init__(self, bot_management: AsyncBotManagementResource) -> None:
        self._bot_management = bot_management

        self.update = async_to_streamed_response_wrapper(
            bot_management.update,
        )
        self.get = async_to_streamed_response_wrapper(
            bot_management.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["LiveInputListResponse", "LiveInput"]


class LiveInput(BaseModel):
    created: Optional[datetime] = None
    """The date and time the live input was created."""

    delete_recording_after_days: Optional[float] = FieldInfo(alias="deleteRecordingAfterDays", default=None)
    """
    Indicates the number of days after which the live inputs recordings will be
    deleted. When a stream completes and the recording is ready, the value is used
    to calculate a scheduled deletion date for that recording. Omit the field to
    indicate no change, or include with a `null` value to remove an existing
    scheduled deletion.
    """

    enabled: Optional[bool] = None
    """Indicates whether the live input is enabled and can accept streams."""

    meta: Optional[object] = None
    """
    A user modifiable key-value store used to reference other systems of record for
    managing live inputs.
    """

    modified: Optional[datetime] = None
    """The date and time the live input was last modified."""

    uid: Optional[str] = None
    """A unique identifier for a live input."""


class LiveInputListResponse(BaseModel):
    live_inputs: Optional[List[LiveInput]] = FieldInfo(alias="liveInputs", default=None)

    range: Optional[int] = None
    """The total number of remaining live inputs based on cursor position."""

    total: Optional[int] = None
    """The total number of live inputs that match the provided filters."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SnippetListParams"]


class SnippetListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Use this field to specify the unique ID of the zone."""

    page: int
    """Specifies the current page number."""

    per_page: int
    """Specifies how many results to return per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.browser_rendering.devtools.browser import target_create_params
from .....types.browser_rendering.devtools.browser.target_get_response import TargetGetResponse
from .....types.browser_rendering.devtools.browser.target_list_response import TargetListResponse
from .....types.browser_rendering.devtools.browser.target_close_response import TargetCloseResponse
from .....types.browser_rendering.devtools.browser.target_create_response import TargetCreateResponse
from .....types.browser_rendering.devtools.browser.target_activate_response import TargetActivateResponse

__all__ = ["TargetsResource", "AsyncTargetsResource"]


class TargetsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TargetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TargetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TargetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TargetsResourceWithStreamingResponse(self)

    def create(
        self,
        session_id: str,
        *,
        account_id: str,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TargetCreateResponse:
        """Opens a new tab in the browser.

        Optionally specify a URL to navigate to.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/json/new",
                account_id=account_id,
                session_id=session_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"url": url}, target_create_params.TargetCreateParams),
            ),
            cast_to=TargetCreateResponse,
        )

    def list(
        self,
        session_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TargetListResponse:
        """
        Returns a list of all debuggable targets including tabs, pages, service workers,
        and other browser contexts.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/json/list",
                account_id=account_id,
                session_id=session_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TargetListResponse,
        )

    def activate(
        self,
        target_id: str,
        *,
        account_id: str,
        session_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TargetActivateResponse:
        """
        Activates (brings to front) a specific browser target by its ID.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          target_id: Target ID to activate.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        if not target_id:
            raise ValueError(f"Expected a non-empty value for `target_id` but received {target_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/json/activate/{target_id}",
                account_id=account_id,
                session_id=session_id,
                target_id=target_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TargetActivateResponse,
        )

    def close(
        self,
        target_id: str,
        *,
        account_id: str,
        session_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TargetCloseResponse:
        """Closes a specific browser target (tab, page, etc.) by its ID.

        Returns 'Target is
        closing' on success or an error if the target is not found.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          target_id: Target ID to close.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        if not target_id:
            raise ValueError(f"Expected a non-empty value for `target_id` but received {target_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/json/close/{target_id}",
                account_id=account_id,
                session_id=session_id,
                target_id=target_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TargetCloseResponse,
        )

    def get(
        self,
        target_id: str,
        *,
        account_id: str,
        session_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TargetGetResponse:
        """
        Returns the debuggable target with the given ID.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          target_id: Target ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        if not target_id:
            raise ValueError(f"Expected a non-empty value for `target_id` but received {target_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/json/list/{target_id}",
                account_id=account_id,
                session_id=session_id,
                target_id=target_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TargetGetResponse,
        )


class AsyncTargetsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTargetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTargetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTargetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTargetsResourceWithStreamingResponse(self)

    async def create(
        self,
        session_id: str,
        *,
        account_id: str,
        url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TargetCreateResponse:
        """Opens a new tab in the browser.

        Optionally specify a URL to navigate to.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/json/new",
                account_id=account_id,
                session_id=session_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"url": url}, target_create_params.TargetCreateParams),
            ),
            cast_to=TargetCreateResponse,
        )

    async def list(
        self,
        session_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TargetListResponse:
        """
        Returns a list of all debuggable targets including tabs, pages, service workers,
        and other browser contexts.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/json/list",
                account_id=account_id,
                session_id=session_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TargetListResponse,
        )

    async def activate(
        self,
        target_id: str,
        *,
        account_id: str,
        session_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TargetActivateResponse:
        """
        Activates (brings to front) a specific browser target by its ID.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          target_id: Target ID to activate.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        if not target_id:
            raise ValueError(f"Expected a non-empty value for `target_id` but received {target_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/json/activate/{target_id}",
                account_id=account_id,
                session_id=session_id,
                target_id=target_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TargetActivateResponse,
        )

    async def close(
        self,
        target_id: str,
        *,
        account_id: str,
        session_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TargetCloseResponse:
        """Closes a specific browser target (tab, page, etc.) by its ID.

        Returns 'Target is
        closing' on success or an error if the target is not found.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          target_id: Target ID to close.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        if not target_id:
            raise ValueError(f"Expected a non-empty value for `target_id` but received {target_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/json/close/{target_id}",
                account_id=account_id,
                session_id=session_id,
                target_id=target_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TargetCloseResponse,
        )

    async def get(
        self,
        target_id: str,
        *,
        account_id: str,
        session_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TargetGetResponse:
        """
        Returns the debuggable target with the given ID.

        Args:
          account_id: Account ID.

          session_id: Browser session ID.

          target_id: Target ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not session_id:
            raise ValueError(f"Expected a non-empty value for `session_id` but received {session_id!r}")
        if not target_id:
            raise ValueError(f"Expected a non-empty value for `target_id` but received {target_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/devtools/browser/{session_id}/json/list/{target_id}",
                account_id=account_id,
                session_id=session_id,
                target_id=target_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TargetGetResponse,
        )


class TargetsResourceWithRawResponse:
    def __init__(self, targets: TargetsResource) -> None:
        self._targets = targets

        self.create = to_raw_response_wrapper(
            targets.create,
        )
        self.list = to_raw_response_wrapper(
            targets.list,
        )
        self.activate = to_raw_response_wrapper(
            targets.activate,
        )
        self.close = to_raw_response_wrapper(
            targets.close,
        )
        self.get = to_raw_response_wrapper(
            targets.get,
        )


class AsyncTargetsResourceWithRawResponse:
    def __init__(self, targets: AsyncTargetsResource) -> None:
        self._targets = targets

        self.create = async_to_raw_response_wrapper(
            targets.create,
        )
        self.list = async_to_raw_response_wrapper(
            targets.list,
        )
        self.activate = async_to_raw_response_wrapper(
            targets.activate,
        )
        self.close = async_to_raw_response_wrapper(
            targets.close,
        )
        self.get = async_to_raw_response_wrapper(
            targets.get,
        )


class TargetsResourceWithStreamingResponse:
    def __init__(self, targets: TargetsResource) -> None:
        self._targets = targets

        self.create = to_streamed_response_wrapper(
            targets.create,
        )
        self.list = to_streamed_response_wrapper(
            targets.list,
        )
        self.activate = to_streamed_response_wrapper(
            targets.activate,
        )
        self.close = to_streamed_response_wrapper(
            targets.close,
        )
        self.get = to_streamed_response_wrapper(
            targets.get,
        )


class AsyncTargetsResourceWithStreamingResponse:
    def __init__(self, targets: AsyncTargetsResource) -> None:
        self._targets = targets

        self.create = async_to_streamed_response_wrapper(
            targets.create,
        )
        self.list = async_to_streamed_response_wrapper(
            targets.list,
        )
        self.activate = async_to_streamed_response_wrapper(
            targets.activate,
        )
        self.close = async_to_streamed_response_wrapper(
            targets.close,
        )
        self.get = async_to_streamed_response_wrapper(
            targets.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["RuleUpdateParams", "Rule"]


class RuleUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    rules: Required[Iterable[Rule]]


class Rule(TypedDict, total=False):
    action: Required[Literal["bypass_waiting_room"]]
    """The action to take when the expression matches."""

    expression: Required[str]
    """Criteria defining when there is a match for the current rule."""

    description: str
    """The description of the rule."""

    enabled: bool
    """When set to true, the rule is enabled."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["Integration", "Config"]


class Config(BaseModel):
    """The configuration object containing third-party integration information."""

    api_url: str
    """The Workspace One API URL provided in the Workspace One Admin Dashboard."""

    auth_url: str
    """The Workspace One Authorization URL depending on your region."""

    client_id: str
    """The Workspace One client ID provided in the Workspace One Admin Dashboard."""


class Integration(BaseModel):
    id: Optional[str] = None
    """API UUID."""

    config: Optional[Config] = None
    """The configuration object containing third-party integration information."""

    interval: Optional[str] = None
    """The interval between each posture check with the third-party API.

    Use `m` for minutes (e.g. `5m`) and `h` for hours (e.g. `12h`).
    """

    name: Optional[str] = None
    """The name of the device posture integration."""

    type: Optional[
        Literal[
            "workspace_one",
            "crowdstrike_s2s",
            "uptycs",
            "intune",
            "kolide",
            "tanium_s2s",
            "sentinelone_s2s",
            "custom_s2s",
        ]
    ] = None
    """The type of device posture integration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RayIDGetParams"]


class RayIDGetParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    fields: str
    """
    The `/received` route by default returns a limited set of fields, and allows
    customers to override the default field set by specifying individual fields. The
    reasons for this are: 1. Most customers require only a small subset of fields,
    but that subset varies from customer to customer; 2. Flat schema is much easier
    to work with downstream (importing into BigTable etc); 3. Performance (time to
    process, file size). If `?fields=` is not specified, default field set is
    returned. This default field set may change at any time. When `?fields=` is
    provided, each record is returned with the specified fields. `fields` must be
    specified as a comma separated list without any whitespaces, and all fields must
    exist. The order in which fields are specified does not matter, and the order of
    fields in the response is not specified.
    """

    timestamps: Literal["unix", "unixnano", "rfc3339"]
    """By default, timestamps in responses are returned as Unix nanosecond integers.

    The `?timestamps=` argument can be set to change the format in which response
    timestamps are returned. Possible values are: `unix`, `unixnano`, `rfc3339`.
    Note that `unix` and `unixnano` return timestamps as integers; `rfc3339` returns
    timestamps as strings.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .services import (
    ServicesResource,
    AsyncServicesResource,
    ServicesResourceWithRawResponse,
    AsyncServicesResourceWithRawResponse,
    ServicesResourceWithStreamingResponse,
    AsyncServicesResourceWithStreamingResponse,
)
from .directory import (
    DirectoryResource,
    AsyncDirectoryResource,
    DirectoryResourceWithRawResponse,
    AsyncDirectoryResourceWithRawResponse,
    DirectoryResourceWithStreamingResponse,
    AsyncDirectoryResourceWithStreamingResponse,
)

__all__ = [
    "ServicesResource",
    "AsyncServicesResource",
    "ServicesResourceWithRawResponse",
    "AsyncServicesResourceWithRawResponse",
    "ServicesResourceWithStreamingResponse",
    "AsyncServicesResourceWithStreamingResponse",
    "DirectoryResource",
    "AsyncDirectoryResource",
    "DirectoryResourceWithRawResponse",
    "AsyncDirectoryResourceWithRawResponse",
    "DirectoryResourceWithStreamingResponse",
    "AsyncDirectoryResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .host import Host
from ..._models import BaseModel

__all__ = ["Header"]


class Header(BaseModel):
    """The request header is used to pass additional information with an HTTP request.

    Currently supported header is 'Host'.
    """

    host: Optional[List[Host]] = FieldInfo(alias="Host", default=None)
    """The 'Host' header allows to override the hostname set in the HTTP request.

    Current support is 1 'Host' header override per origin.
    """
